@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
+import datetime
 import logging
 import os
+import warnings
 from collections import defaultdict
 from io import BytesIO
-from typing import List, Optional
+from typing import List, Optional, Union
+
+import async_retriever as ar
+import dateutil.parser as parser
 
 try:
     from typing import Literal
@@ -28,28 +33,36 @@ _settings_map = defaultdict(lambda: [{"metric": "", "english": ""}])
 # Preliminary or verified water levels, depending on availability.
 _settings_map["water_level"] = [
     {"metric": "m", "english": "ft"},
-    "h",
+    "6T",
+    ["Inferred", "FlatTol", "RateTol", "TempRateTol"],
 ]
 
 # Air temperature as measured at the station.
 _settings_map["air_temperature"] = [
     {"metric": "degC", "english": "degF"},
     "h",
+    ["MaxTol", "MinTol", "RateTol"],
 ]
 
 # Water temperature as measured at the station.
 _settings_map["water_temperature"] = [
     {"metric": "degC", "english": "degF"},
     "h",
+    ["MaxTol", "MinTol", "RateTol"],
 ]
 
 # Wind speed, direction, and gusts as measured at the station.
-_settings_map["wind"] = [{"metric": "m/s", "english": "ft/s"}, "h"]
+_settings_map["wind"] = [
+    {"metric": "m/s", "english": "ft/s"},
+    "h",
+    ["MaxTol", "RateTol"],
+]
 
 # Barometric pressure as measured at the station.
 _settings_map["air_pressure"] = [
     {"metric": "mb", "english": "mb"},
     "h",
+    ["MaxTol", "MinTol", "RateTol"],
 ]
 
 # Air Gap (distance between a bridge and the water's surface) at the
@@ -57,12 +70,14 @@ _settings_map["air_pressure"] = [
 _settings_map["air_gap"] = [
     {"metric": "m", "english": "ft"},
     "h",
+    [""],
 ]
 
 # The water's conductivity as measured at the station.
 _settings_map["conductivity"] = [
     {"metric": "mS/cm", "english": "mS/cm"},
     "h",
+    ["MaxTol", "MinTol", "RateTol"],
 ]
 
 # Visibility from the station's visibility sensor. A measure of atmospheric
@@ -70,66 +85,77 @@ _settings_map["conductivity"] = [
 _settings_map["visibility"] = [
     {"metric": "km", "english": "nm"},
     "h",
+    [""],
 ]
 
 # Relative humidity as measured at the station.
 _settings_map["humidity"] = [
     {"metric": "percent", "english": "percent"},
     "h",
+    ["MaxTol", "MinTol", "RateTol"],
 ]
 
 # Salinity and specific gravity data for the station.
 _settings_map["salinity"] = [
     {"metric": "PSU", "english": "PSU"},
     "h",
+    [""],
 ]
 
 # Verified hourly height water level data for the station.
 _settings_map["hourly_height"] = [
     {"metric": "m", "english": "ft"},
     "h",
+    ["Inferred", "LimitTol"],
 ]
 
 # Verified high/low water level data for the station.
 _settings_map["high_low"] = [
     {"metric": "m", "english": "ft"},
     None,
+    ["Inferred", "LimitTol"],
 ]
 
 # Verified daily mean water level data for the station.
 _settings_map["daily_mean"] = [
     {"metric": "m", "english": "ft"},
     None,
+    ["Inferred", "LimitTol"],
 ]
 
 # Verified monthly mean water level data for the station.
 _settings_map["monthly_mean"] = [
     {"metric": "m", "english": "ft"},
     None,
+    [""],
 ]
 
 # One minute water level data for the station.
 _settings_map["one_minute_water_level"] = [
     {"metric": "m", "english": "ft"},
     None,
+    [""],
 ]
 
 # 6 minute predictions water level data for the station.
 _settings_map["predictions"] = [
     {"metric": "m", "english": "ft"},
     "h",
+    [""],
 ]
 
 # datums data for the currents stations.
 _settings_map["datums"] = [
     {"metric": "m", "english": "ft"},
     None,
+    [""],
 ]
 
 # Currents data for currents stations.
 _settings_map["currents"] = [
     {"metric": "m/s", "english": "ft/s"},
     "h",
+    [""],
 ]
 
 
@@ -141,13 +167,13 @@ def coops_cli(
     begin_date=None,
     end_date=None,
     range=None,
-    product="water_level",
+    product="hourly_height",
     datum="NAVD",
     time_zone="GMT",
     interval="h",
     bin=None,
 ):
-    r"""station: Center for Operational Oceanographic Products and Services
+    r"""global station 1T,6T,H,D,M: Center for Operational Oceanographic Products and Services
 
     CO-OPS web services is at https://tidesandcurrents.noaa.gov/api/.
     The time zone of the returned data depends on the setting of the
@@ -156,10 +182,7 @@ def coops_cli(
     Parameters
     ----------
     station
-        A 7 character station ID, or a currents station ID.  Specify the
-        station ID with the "station=" parameter.::
-
-            Example: '--station=9414290'
+        A 7 character station ID, or a currents station ID.
 
         Station listings for various products can be viewed at
         https://tidesandcurrents.noaa.gov or viewed on a map at Tides
@@ -253,7 +276,7 @@ def coops_cli(
         related parameters.
 
     product : str
-        [optional, default is 'water_level']
+        [optional, default is 'hourly_height']
 
         Specify the observation requested.
 
@@ -625,7 +648,7 @@ def coops_cli(
         bin.
 
     """
-    tsutils._printiso(
+    tsutils.printiso(
         coops(
             station,
             date=date,
@@ -641,108 +664,25 @@ def coops_cli(
     )
 
 
-def core(params):
-    if params["end_date"] is not None:
-        params["end_date"] = params["end_date"].strftime("%Y%m%d")
-    if params["begin_date"] is not None:
-        params["begin_date"] = params["begin_date"].strftime("%Y%m%d")
-
-    url = r"https://tidesandcurrents.noaa.gov/api/datagetter"
-    session = utils.requests_retry_session()
-    req = session.get(url, params=params)
-
-    if os.path.exists("debug_tsgettoolbox"):
-        logging.warning(req.url)
-    req.raise_for_status()
-
-    if (
-        b"Error" in req.content
-        or b"Wrong" in req.content
-        or b"Range limit" in req.content
-    ):
-        df = pd.DataFrame()
-        error = req.content
-    else:
-        df = pd.read_csv(BytesIO(req.content), index_col=0, parse_dates=True)
-        error = ""
-    return df, error
-
-
-def precore(query_params, i):
-    if query_params["date"] is not None:
-        ndf, error = core(query_params)
-    elif (
-        query_params["range"] is not None
-        and query_params["begin_date"] is None
-        and query_params["end_date"] is None
-    ):
-        ndf, error = core(query_params)
-    elif query_params["begin_date"] is not None and query_params["range"] is not None:
-        ndf, error = core(query_params)
-    elif (
-        query_params["begin_date"] is not None and query_params["end_date"] is not None
-    ):
-        ndf = pd.DataFrame()
-        sdate = query_params["begin_date"]
-        edate = query_params["end_date"]
-        testdate = sdate
-        while testdate < edate:
-            query_params["begin_date"] = testdate
-            testdate = testdate + pd.Timedelta(days=deltas[i])
-            if testdate > edate:
-                testdate = edate
-            query_params["end_date"] = testdate
-            df, error = core(query_params)
-            ndf = ndf.combine_first(df)
-    elif query_params["end_date"] is not None and query_params["range"] is not None:
-        ndf, error = core(query_params)
-
-    if len(ndf) == 0:
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-COOPS service returned the error "{}".
-""".format(
-                    error
-                )
-            )
-        )
-
-    new_column_names = []
-    for icolumn_name in ndf.columns:
-        ncolumn_name = icolumn_name.lower().strip().replace(" ", "_")
-        units = _settings_map[ncolumn_name][0][query_params["units"]]
-        unitstr = ncolumn_name
-        if units != "":
-            unitstr = ":".join([ncolumn_name, units])
-        new_column_names.append(("NOS", query_params["station"], unitstr))
-    ndf.columns = ["-".join(i).rstrip("-") for i in new_column_names]
-    time_zone_name = query_params["time_zone"].upper()
-    if time_zone_name == "GMT":
-        time_zone_name = "UTC"
-    ndf = ndf.tz_localize(time_zone_name)
-    ndf.index.name = "Datetime:{}".format(time_zone_name)
-    return ndf
-
-
 deltas = {
     "water_level": 31,
-    "air_temperature": 31,
-    "water_temperature": 31,
-    "wind": 31,
+    "air_temperature": 365,
+    "water_temperature": 365,
+    "wind": 365,
     "air_gap": 31,
-    "conductivity": 31,
-    "visibility": 31,
-    "humidity": 31,
+    "conductivity": 365,
+    "visibility": 365,
+    "humidity": 365,
     "salinity": 31,
     "hourly_height": 365,
     "high_low": 365,
     "daily_mean": 3650,
     "monthly_mean": 3650,
-    "one_minute_water_level": 31,
+    "one_minute_water_level": 4,
     "predictions": 31,
     "datums": 31,
     "currents": 31,
+    "air_pressure": 365,
 }
 
 
@@ -760,58 +700,212 @@ def coops(
     begin_date: Optional[pd.Timestamp] = None,
     end_date: Optional[pd.Timestamp] = None,
     range: Optional[int] = None,
-    product: List[
-        Literal[
-            "water_level",
-            "air_temperature",
-            "water_temperature",
-            "wind",
-            "air_gap",
-            "conductivity",
-            "visibility",
-            "humidity",
-            "salinity",
-            "hourly_height",
-            "high_low",
-            "daily_mean",
-            "monthly_mean",
-            "one_minute_water_level",
-            "predictions",
-            "datums",
-            "currents",
-        ]
-    ] = "water_level",
+    product: Union[
+        List[
+            Literal[
+                "water_level",
+                "air_temperature",
+                "water_temperature",
+                "wind",
+                "air_gap",
+                "conductivity",
+                "visibility",
+                "humidity",
+                "salinity",
+                "hourly_height",
+                "high_low",
+                "daily_mean",
+                "monthly_mean",
+                "one_minute_water_level",
+                "predictions",
+                "datums",
+                "currents",
+                "air_pressure",
+            ]
+        ],
+        str,
+    ] = "hourly_height",
     datum: Literal["MHHW", "MHW", "MTL", "MSL", "MLW", "MLLW", "NAVD", "STND"] = "NAVD",
     time_zone: Literal["GMT", "UTC", "LST", "LST_LDT"] = "GMT",
     interval="h",
     bin=None,
 ):
     r"""Download Center for Operational Oceanographic Products and Services."""
+    disable_caching = False
     params = {}
     params["station"] = station
-    params["date"] = date
-    params["range"] = range
     params["product"] = product
     params["bin"] = bin
     params["interval"] = interval
     params["units"] = "metric"
     params["time_zone"] = time_zone
     params["datum"] = datum
-    try:
-        params["begin_date"] = begin_date
-    except ValueError:
-        params["begin_date"] = None
-    try:
-        params["end_date"] = end_date
-    except ValueError:
-        params["end_date"] = None
-    params["format"] = "csv"
+    params["format"] = "json"
     params["application"] = "tsgettoolbox"
 
+    if date is not None:
+        begin_date = None
+        end_date = None
+        range = None
+        disable_caching = True
+    if range is not None and begin_date is None and end_date is None:
+        disable_caching = True
+    if begin_date is not None and range is not None:
+        end_date = None
+    if begin_date is not None and end_date is not None:
+        range = None
+
+    # Normalize begin_data and end_date to not extend beyond the "established"
+    # and "removed" dates of the station.
+    if (date is None) and (range is None):
+        station_data = ar.retrieve_json(
+            [
+                f"https://api.tidesandcurrents.noaa.gov/mdapi/prod/webapi/stations/{station}/details.json"
+            ],
+            [{"params": {"expand": "detail", "units": "english"}}],
+        )
+        test_begin = pd.Timestamp(station_data[0]["established"])
+        if (begin_date is None) or (begin_date < test_begin):
+            begin_date = test_begin
+        test_end = pd.Timestamp(station_data[0]["removed"])
+        if test_end is pd.NaT:
+            test_end = datetime.datetime.utcnow()
+        if (end_date is None) or (end_date > test_end):
+            end_date = test_end
+
+    params["date"] = date
+    params["begin_date"] = begin_date
+    params["end_date"] = end_date
+    params["range"] = range
+
+    if params["begin_date"] is not None:
+        params["begin_date"] = params["begin_date"].strftime("%Y%m%d")
+
+    if params["end_date"] is not None:
+        params["end_date"] = params["end_date"].strftime("%Y%m%d")
+
+    if len(product) > 1 and "datums" in product:
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+The "datums" product is not a time-series and can't be mixed with other products."
+"""
+            )
+        )
+
     ndf = pd.DataFrame()
-    for cnt, i in enumerate(product):
-        r = precore(params, i)
-        ndf = ndf.join(r, how="outer", rsuffix="_{}".format(cnt))
+    for produc in product:
+        if produc == "datums":
+            params["range"] = 1
+            params["begin_date"] = None
+            params["end_date"] = None
+
+        periods = []
+        if params["begin_date"] is not None and params["end_date"] is not None:
+            delta = datetime.timedelta(days=deltas[produc])
+            period_start = parser.parse(params["begin_date"])
+            while period_start < parser.parse(params["end_date"]):
+                period_end = min(period_start + delta, parser.parse(params["end_date"]))
+                periods.append((period_start, period_end))
+                period_start = period_end
+
+        urls = [r"https://tidesandcurrents.noaa.gov/api/datagetter"]
+        kwds = [{"params": params}]
+
+        if params["begin_date"] is not None and params["end_date"] is not None:
+            urls, kwds = zip(
+                *[
+                    (
+                        r"https://tidesandcurrents.noaa.gov/api/datagetter",
+                        {
+                            "params": {
+                                "station": station,
+                                "date": date,
+                                "range": range,
+                                "product": produc,
+                                "bin": bin,
+                                "interval": interval,
+                                "units": "metric",
+                                "time_zone": time_zone,
+                                "datum": datum,
+                                "begin_date": s.strftime("%Y%m%d"),
+                                "end_date": e.strftime("%Y%m%d"),
+                                "format": "json",
+                                "application": "tsgettoolbox",
+                            }
+                        },
+                    )
+                    for s, e in periods
+                ]
+            )
+
+        kwds = [
+            {"params": {k: v for k, v in i["params"].items() if v is not None}}
+            for i in kwds
+        ]
+        resp = ar.retrieve_json(urls, kwds, disable=disable_caching)
+        if produc in ["datums"]:
+            resp = [i["datums"] for i in resp if "datums" in i]
+        elif produc in ["predictions"]:
+            resp = [i["predictions"] for i in resp if "predictions" in i]
+        else:
+            resp = [i["data"] for i in resp if "data" in i]
+
+        if not resp:
+            warnings.warn(
+                tsutils.error_wrapper(
+                    """
+No data for this product and time frame at this station.
+"""
+                )
+            )
+            return pd.DataFrame()
+
+        resp = [item for sublist in resp for item in sublist]
+        resp = pd.DataFrame(resp)
+        if produc in ["datums"]:
+            resp = resp.set_index("n")
+        elif produc in ["monthly_mean"]:
+            resp["t"] = pd.to_datetime(resp[["year", "month"]].assign(Day=1))
+            resp = resp.drop(["year", "month"], axis="columns")
+            resp = resp.set_index("t")
+        else:
+            resp = resp.set_index("t")
+
+        time_zone_name = params["time_zone"].upper()
+        if time_zone_name == "GMT":
+            time_zone_name = "UTC"
+        resp.index.name = "Datetime:{}".format(time_zone_name)
+
+        if produc in ["datums"]:
+            resp.index.name = "Datums"
+
+        if "f" in resp.columns:
+            addcols = resp["f"].str.split(pat=",", expand=True)
+            resp = resp.drop("f", axis="columns")
+            addcols.columns = _settings_map[produc][2]
+            resp = resp.join(addcols)
+
+        units = _settings_map[produc][0][params["units"]]
+        if produc in ["wind"]:
+            rename_cols = {
+                "s": f"wind_speed:{units}",
+                "g": f"wind_gust:{units}",
+            }
+            resp = resp.rename(rename_cols, axis="columns")
+        rename_cols = {
+            "v": f"{produc}:{units}",
+            "s": f"{produc}_sigma:{units}",
+            "q": f"{produc}_quality",
+            "Inferred": f"{produc}_Inferred",
+            "MaxTol": f"{produc}_MaxTol",
+            "MinTol": f"{produc}_MinTol",
+            "LimitTol": f"{produc}_LimitTol",
+            "TempRateTol": f"{produc}_TempRateTol",
+            "RateTol": f"{produc}_RateTol",
+        }
+        resp = resp.rename(rename_cols, axis="columns")
+        ndf = ndf.join(resp, how="outer")
     return ndf
 
 
@@ -827,8 +921,7 @@ if __name__ == "__main__":
     r = coops(
         station="8720218",
         product="water_level",
-        interval="h",
-        time_zone="gmt",
+        time_zone="lst",
         datum="mllw",
         range=20,
         begin_date=None,
@@ -836,7 +929,7 @@ if __name__ == "__main__":
         date=None,
     )
 
-    print("tidesandcurrents")
+    print("tidesandcurrents 1 range=20")
     print(r)
 
     r = coops(
@@ -851,7 +944,7 @@ if __name__ == "__main__":
         date=None,
     )
 
-    print("tidesandcurrents")
+    print("tidesandcurrents 2 begin_date 01/10/2002 and range=2")
     print(r)
 
     r = coops(
@@ -866,7 +959,7 @@ if __name__ == "__main__":
         date=None,
     )
 
-    print("tidesandcurrents")
+    print("tidesandcurrents 3 begin_date 01/10/2002 and range=5")
     print(r)
 
     r = coops(
@@ -881,7 +974,7 @@ if __name__ == "__main__":
         date=None,
     )
 
-    print("tidesandcurrents")
+    print("tidesandcurrents 4 range=3 end_date-01/10/2002")
     print(r)
 
     r = coops(
@@ -896,7 +989,7 @@ if __name__ == "__main__":
         date=None,
     )
 
-    print("tidesandcurrents")
+    print("tidesandcurrents 5 begin_date=01/10/2002 end_date=2003/01/01")
     print(r)
 
     try:
@@ -913,3 +1006,33 @@ if __name__ == "__main__":
         print(r)
     except ValueError:
         print("Correct ValueError")
+
+    for prod in _settings_map:
+        print(f"!!!!{prod}")
+        r = coops(
+            station="8720218",
+            product=prod,
+            interval="h",
+            time_zone="gmt",
+            datum="mllw",
+            begin_date="01/10/2002",
+            end_date="2003-01-01",
+            range=None,
+            date=None,
+        )
+
+        print(r)
+
+    print("hourly_height and wind")
+    r = coops(
+        station="8720218",
+        product=["hourly_height", "wind"],
+        interval="h",
+        time_zone="gmt",
+        datum="mllw",
+        begin_date="01/10/2002",
+        end_date="2003-01-01",
+        range=None,
+        date=None,
+    )
+    print(r)
