@@ -2,6 +2,7 @@
 """Download data from Florida Automated Weather Network (FAWN)."""
 
 import mando
+import pandas as pd
 
 try:
     from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
@@ -12,7 +13,7 @@ from tstoolbox import tsutils
 
 from tsgettoolbox import utils
 
-_avail_vars = {
+_vars = {
     "pr": {
         "sname": "pr",
         "lname": "precipitation_amount",
@@ -139,12 +140,13 @@ _avail_vars = {
         "standard_name": "spi",
         "vname": "METDATA (GRIDMET) 180 Day Standardized Precipitation Index(SPI) - Pentad",
     },
-    "spi270d": {
-        "sname": "spi",
-        "lname": "standard_precipitation_index_270_day",
-        "standard_name": "spi",
-        "vname": "METDATA (GRIDMET) 270 Day Standardized Precipitation Index(SPI) - Pentad",
-    },
+    # Commented out because it has an unusual URL.
+    #    "spi270d": {
+    #        "sname": "spi",
+    #        "lname": "standard_precipitation_index_270_day",
+    #        "standard_name": "spi",
+    #        "vname": "METDATA (GRIDMET) 270 Day Standardized Precipitation Index(SPI) - Pentad",
+    #    },
     "spi1y": {
         "sname": "spi",
         "lname": "standard_precipitation_index_1_year",
@@ -267,8 +269,7 @@ _avail_vars = {
     },
 }
 
-
-tsutils.docstrings.update({k: v["standard_name"] for k, v in _avail_vars.items()})
+tsutils.docstrings.update({k: f"{v['standard_name']:50}" for k, v in _vars.items()})
 
 
 @mando.command("metdata", formatter_class=HelpFormatter, doctype="numpy")
@@ -280,7 +281,7 @@ def metdata_cli(
     start_date=None,
     end_date=None,
 ):
-    r"""gridded: Download daily data from METDATA.
+    r"""NAmerica 4km 1980- D: Download daily data from METDATA based on PRISM.
 
     This archive contains daily surface meteorological (METDATA) data for the
     Continental United States at 4-km (1/24-deg) resolution. The meteorological
@@ -393,10 +394,8 @@ def metdata_cli(
 
     Parameters
     ----------
-    {lat}
-
-    {lon}
-
+    ${lat}
+    ${lon}
     variables : str
         At the command line can supply a comma separated list of variable
         names.  Using the Python API needs to be a Python list of strings.
@@ -407,94 +406,91 @@ def metdata_cli(
         +----------+----------------------------------------------------+-------+
         | Short    | Long                                               | Units |
         +==========+====================================================+=======+
-        | pr       | {pr:50} | mm    |
+        | pr       | ${pr} | mm    |
         +----------+----------------------------------------------------+-------+
-        | rmax     | {rmax:50} |       |
+        | rmax     | ${rmax} |       |
         +----------+----------------------------------------------------+-------+
-        | rmin     | {rmin:50} |       |
+        | rmin     | ${rmin} |       |
         +----------+----------------------------------------------------+-------+
-        | sph      | {sph:50} | kg/kg |
+        | sph      | ${sph} | kg/kg |
         +----------+----------------------------------------------------+-------+
-        | srad     | {srad:50} | W/m2  |
+        | srad     | ${srad} | W/m2  |
         +----------+----------------------------------------------------+-------+
-        | tmmn     | {tmmn:50} | degK  |
+        | tmmn     | ${tmmn} | degK  |
         +----------+----------------------------------------------------+-------+
-        | tmmx     | {tmmx:50} | degK  |
+        | tmmx     | ${tmmx} | degK  |
         +----------+----------------------------------------------------+-------+
-        | vs       | {vs:50} | m/s   |
+        | vs       | ${vs} | m/s   |
         +----------+----------------------------------------------------+-------+
-        | bi       | {bi:50} |       |
+        | bi       | ${bi} |       |
         +----------+----------------------------------------------------+-------+
-        | fm100    | {fm100:50} |       |
+        | fm100    | ${fm100} |       |
         +----------+----------------------------------------------------+-------+
-        | fm1000   | {fm1000:50} |       |
+        | fm1000   | ${fm1000} |       |
         +----------+----------------------------------------------------+-------+
-        | erc      | {erc:50} |       |
+        | erc      | ${erc} |       |
         +----------+----------------------------------------------------+-------+
-        | pdsi     | {pdsi:50} |       |
+        | pdsi     | ${pdsi} |       |
         +----------+----------------------------------------------------+-------+
-        | etr      | {etr:50} |       |
+        | etr      | ${etr} |       |
         +----------+----------------------------------------------------+-------+
-        | pet      | {pet:50} |       |
+        | pet      | ${pet} |       |
         +----------+----------------------------------------------------+-------+
-        | vpd      | {vpd:50} |       |
+        | vpd      | ${vpd} |       |
         +----------+----------------------------------------------------+-------+
-        | spi14d   | {spi14d:50} |       |
+        | spi14d   | ${spi14d} |       |
         +----------+----------------------------------------------------+-------+
-        | spi30d   | {spi30d:50} |       |
+        | spi30d   | ${spi30d} |       |
         +----------+----------------------------------------------------+-------+
-        | spi90d   | {spi90d:50} |       |
+        | spi90d   | ${spi90d} |       |
         +----------+----------------------------------------------------+-------+
-        | spi180d  | {spi180d:50} |       |
+        | spi180d  | ${spi180d} |       |
         +----------+----------------------------------------------------+-------+
-        | spi270d  | {spi270d:50} |       |
+        | spi1y    | ${spi1y} |       |
         +----------+----------------------------------------------------+-------+
-        | spi1y    | {spi1y:50} |       |
+        | spi2y    | ${spi2y} |       |
         +----------+----------------------------------------------------+-------+
-        | spi2y    | {spi2y:50} |       |
+        | spi5y    | ${spi5y} |       |
         +----------+----------------------------------------------------+-------+
-        | spi5y    | {spi5y:50} |       |
+        | spei14d  | ${spei14d} |       |
         +----------+----------------------------------------------------+-------+
-        | spei14d  | {spei14d:50} |       |
+        | spei30d  | ${spei30d} |       |
         +----------+----------------------------------------------------+-------+
-        | spei30d  | {spei30d:50} |       |
+        | spei90d  | ${spei90d} |       |
         +----------+----------------------------------------------------+-------+
-        | spei90d  | {spei90d:50} |       |
+        | spei180d | ${spei180d} |       |
         +----------+----------------------------------------------------+-------+
-        | spei180d | {spei180d:50} |       |
+        | spei270d | ${spei270d} |       |
         +----------+----------------------------------------------------+-------+
-        | spei270d | {spei270d:50} |       |
+        | spei1y   | ${spei1y} |       |
         +----------+----------------------------------------------------+-------+
-        | spei1y   | {spei1y:50} |       |
+        | spei2y   | ${spei2y} |       |
         +----------+----------------------------------------------------+-------+
-        | spei2y   | {spei2y:50} |       |
+        | spei5y   | ${spei5y} |       |
         +----------+----------------------------------------------------+-------+
-        | spei5y   | {spei5y:50} |       |
+        | eddi14d  | ${eddi14d} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi14d  | {eddi14d:50} |       |
+        | eddi30d  | ${eddi30d} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi30d  | {eddi30d:50} |       |
+        | eddi90d  | ${eddi90d} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi90d  | {eddi90d:50} |       |
+        | eddi180d | ${eddi180d} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi180d | {eddi180d:50} |       |
+        | eddi270d | ${eddi270d} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi270d | {eddi270d:50} |       |
+        | eddi1y   | ${eddi1y} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi1y   | {eddi1y:50} |       |
+        | eddi2y   | ${eddi2y} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi2y   | {eddi2y:50} |       |
+        | eddi5y   | ${eddi5y} |       |
         +----------+----------------------------------------------------+-------+
-        | eddi5y   | {eddi5y:50} |       |
-        +----------+----------------------------------------------------+-------+
-        | z        | {z:50} |       |
+        | z        | ${z} |       |
         +----------+----------------------------------------------------+-------+
 
-    {start_date}
-
-    {end_date}
+    ${start_date}
+    ${end_date}
     """
-    tsutils._printiso(
+    tsutils.printiso(
         metdata(
             lat,
             lon,
@@ -512,40 +508,28 @@ def metdata(
     start_date=None,
     end_date=None,
 ):
-    r"""Download METDATA data from CIDA."""
-
-    turl = [
-        "http://thredds.northwestknowledge.net:8080/thredds/ncss/grid/agg_met_{var}_1979_CurrentYear_CONUS.nc/",
-        "http://thredds.northwestknowledge.net:8080/thredds/ncss/grid/agg_met_{var}_CONUS.nc/",
-    ]
+    r"""Download METDATA data."""
+    if variables is None:
+        vars = list(_vars.keys())
+        vars.sort()
+    else:
+        vars = tsutils.make_list(variables)
 
+    turl = "http://thredds.northwestknowledge.net:8080/thredds/dodsC/agg_met_{}_1979_CurrentYear_CONUS.nc"
     df = utils.opendap(
         turl,
-        variables,
         lat,
         lon,
-        _avail_vars,
+        _vars,
+        variables=variables,
         start_date=start_date,
         end_date=end_date,
-        all_vars_at_url=False,
+        tzname=None,
+        missing_value=-9999,
+        time_name="day",
+        single_var_url=True,
     )
 
-    if len(df.dropna(how="all")) == 0:
-        if start_date is None:
-            start_date = "beginning of record"
-        if end_date is None:
-            end_date = "end of record"
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-GRIDMET returned no data for lat/lon "{lat}/{lon}", variables "{variables}"
-between {start_date} and {end_date}.
-""".format(
-                    **locals()
-                )
-            )
-        )
-
     return df
 
 
@@ -553,5 +537,5 @@ metdata.__doc__ = metdata_cli.__doc__
 
 
 if __name__ == "__main__":
-    r = metdata(29.6, -82.3, "precipitation_amount")
+    r = metdata(29.6, -82.3, variables="pr")
     print(r)
