@@ -13,6 +13,7 @@ try:
 except ImportError:
     from argparse import RawTextHelpFormatter as HelpFormatter
 
+import async_retriever as ar
 from tstoolbox import tsutils
 
 from tsgettoolbox import utils
@@ -151,7 +152,6 @@ _rename = {
 for item in _headermap:
     _rename.update(_headermap[item])
 
-
 _mapnumtoname = {
     0: "Jan",
     1: "Feb",
@@ -190,41 +190,23 @@ def ndbc_to_df(url, **query_params):
         # Yearly
         # https://www.ndbc.noaa.gov/data/historical/stdmet/41012h2012.txt.gz
         filenames.append(
-            "/historical/{3}/{0}{1}{2}.txt.gz".format(
-                query_params["station"], _lmap[table], yr, table
-            )
+            f"/historical/{table}/{query_params['station']}{_lmap[table]}{yr}.txt.gz"
         )
     if edate.year == cyear.year:
         for mnth in range(edate.month):
             # Monthly
             # https://www.ndbc.noaa.gov/data/stdmet/Mar/sauf132018.txt.gz
             filenames.append(
-                "/{3}/{4}/{0}{1}{2}.txt.gz".format(
-                    query_params["station"],
-                    mnth + 1,
-                    yr,
-                    table,
-                    _mapnumtoname[mnth],
-                )
+                f"/{table}/{_mapnumtoname[mnth]}/{query_params['station']}{mnth+1}{yr}.txt.gz"
             )
 
-    for filename in filenames:
-        session = utils.requests_retry_session()
-        req = session.get(url + filename)
-
-        if os.path.exists("debug_tsgettoolbox"):
-            logging.warning(req.url)
-
-        try:
-            req.raise_for_status()
-        except requests.exceptions.HTTPError:
-            continue
-
-        content = BytesIO(req.content)
-        content = GzipFile(fileobj=content).read()
-
-        # Test to see if 1 or 2 line header
-        f = StringIO(content.decode("utf-8"))
+    resp = ar.retrieve_binary([url + i for i in filenames], [{}] * len(filenames))
+    resp = [BytesIO(i) for i in resp]
+    resp = [GzipFile(fileobj=i).read() for i in resp]
+    skiprows = []
+    parse_dates = []
+    for r in resp:
+        f = StringIO(r.decode("utf-8"))
         head1 = f.readline()
         head2 = f.readline()
         f.seek(0)
@@ -237,7 +219,7 @@ def ndbc_to_df(url, **query_params):
         except ValueError:
             skiprows = [1]
 
-        if "mm" == words1[4]:
+        if words1[4] == "mm":
             parse_dates = {"datetime": [0, 1, 2, 3, 4]}
         else:
             parse_dates = {"datetime": [0, 1, 2, 3]}
@@ -255,7 +237,7 @@ def ndbc_to_df(url, **query_params):
 
         if len(tdf) > 0:
             tdf.rename(columns=_rename, inplace=True)
-            df = df.append(tdf)
+            df = pd.concat([df, tdf])
 
     if len(df) == 0:
         raise ValueError(
@@ -285,7 +267,7 @@ No data collected/available within this time frame.
 
 @mando.command("ndbc", formatter_class=HelpFormatter, doctype="numpy")
 def ndbc_cli(station, table, startUTC, endUTC):
-    r"""station: Download historical from the National Data Buoy Center.
+    r"""US station T,6T,10T,15T,H,D:Download historical from the National Data Buoy Center.
 
     Download historical data from the National Data Buoy Center.
 
@@ -524,7 +506,6 @@ def ndbc_cli(station, table, startUTC, endUTC):
         (only seconds are optional)
 
     """
-
     extra_docstring = """
         Derived Met Values
 
@@ -814,12 +795,11 @@ def ndbc_cli(station, table, startUTC, endUTC):
         | REMCAP   | Remaining Battery Capacity (ampere-hours)            |
         +----------+------------------------------------------------------+
         """
-    tsutils._printiso(ndbc(station, table, startUTC, endUTC))
+    tsutils.printiso(ndbc(station, table, startUTC, endUTC))
 
 
 def ndbc(station, table, startUTC, endUTC):
     r"""Download historical from the National Data Buoy Center."""
-
     df = ndbc_to_df(
         r"https://www.ndbc.noaa.gov/data/",
         table=table,
