@@ -231,14 +231,14 @@ def remove_values(
                 values_df = store[values_path]
                 original_datetimes = set(values_df.dropna(how="all").index.tolist())
                 datetimes_to_remove = original_datetimes.intersection(set(datetimes))
-                if not len(datetimes_to_remove):
+                if not datetimes_to_remove:
                     core.log.info(
                         "No %s values matching the given datetimes to remove were found."
                         % variable_code
                     )
                     continue
                 else:
-                    values_df.ix[list(datetimes_to_remove), "value"] = np.nan
+                    values_df.loc[list(datetimes_to_remove), "value"] = np.nan
                     core.log.info(
                         "%i %s values were set to NaNs in file"
                         % (len(datetimes_to_remove), variable_code)
@@ -444,7 +444,7 @@ def update_site_data(
         input_file=input_file,
         methods=methods,
     )
-    if not len(new_site_data):
+    if not new_site_data:
         core.log.info("No new data was found")
         return None
 
@@ -593,9 +593,7 @@ def _nest_dataframe_dicts(unnested_df, nested_column, keys):
 
 def _ptrepack(src, dst, complevel, complib):
     """run ptrepack to repack from src to dst"""
-
     # check_output(['ptrepack','--complevel=%s' % complevel, '--complib=%s' % complib, src, dst])
-
     # fix for for pytables not finding files on windows because of drive in path
     src = os.path.splitdrive(src)[-1]
     dst = os.path.splitdrive(dst)[-1]
