@@ -3,7 +3,11 @@ import os
 import pandas
 import test_util
 
-from ulmo.lcra.waterquality import get_historical_data, get_recent_data, get_sites
+from tsgettoolbox.ulmo.lcra.waterquality import (
+    get_historical_data,
+    get_recent_data,
+    get_sites,
+)
 
 
 def test_get_sites():
@@ -23,7 +27,7 @@ def test_get_historical_data():
 
     assert len(results) == 12
     for data in results:
-        assert data["Site"] == u"12147"
+        assert data["Site"] == "12147"
 
 
 def test_get_recent_data():
