@@ -2,7 +2,7 @@ import datetime
 
 import test_util
 
-import ulmo
+from tsgettoolbox import ulmo
 
 
 def test_get_stations():
@@ -15,7 +15,7 @@ def test_get_stations():
 
 def test_get_station_parameters():
     test_sets = [
-        ("CE7F42E6", {"HP": u"Pool Level (Ft)", "PC": u"Cumulative Precipitation (In)"})
+        ("CE7F42E6", {"HP": "Pool Level (Ft)", "PC": "Cumulative Precipitation (In)"})
     ]
 
     for station_code, test_value in test_sets:
